use actix_web::{error::ErrorInternalServerError, web, Responder};
use sea_orm::{ActiveModelTrait, ActiveValue, EntityTrait, IntoActiveModel};
use serde::{Deserialize, Serialize};

use crate::{auth::AuthedUser, AppState};

use super::user::UserWithoutPassword;

#[derive(Serialize)]
struct GroupWithLicenses {
    #[serde(flatten)]
    group: entity::license_group::Model,
    licenses: Vec<entity::license::Model>,
}

pub struct LicenseController;

impl LicenseController {
    pub async fn list_groups(
        state: web::Data<AppState>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let groups = entity::prelude::LicenseGroup::find()
            .find_with_related(entity::prelude::License)
            .all(db)
            .await
            .map_err(ErrorInternalServerError)?
            .into_iter()
            .map(|(group, licenses)| GroupWithLicenses { group, licenses })
            .collect::<Vec<_>>();
        Ok(web::Json(groups))
    }

    pub async fn create_group(
        state: web::Data<AppState>,
        group: web::Json<entity::license_group::Model>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let group = group.into_inner();
        let mut group = group.into_active_model();
        group.id = ActiveValue::NotSet;

        let res = group.insert(db).await.map_err(ErrorInternalServerError)?;

        Ok(web::Json(res))
    }
    pub async fn create_license(
        state: web::Data<AppState>,
        license: web::Json<entity::license::Model>,
        _executor: AuthedUser,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let license = license.into_inner();
        let mut license = license.into_active_model();
        license.id = ActiveValue::NotSet;

        let res = license.insert(db).await.map_err(ErrorInternalServerError)?;

        Ok(web::Json(res))
    }
}
